@@ -68,6 +68,7 @@ class Host(object):
         self.ports = {}
         self.extraports = {}
         self.os = []
+        self.script_results = []
 
     def get_id(self):
         """Return an id that is used to determine if hosts are "the same" across
@@ -173,6 +174,12 @@ class Host(object):
                 os_elem.appendChild(self.os_to_dom_fragment(document, os))
             elem.appendChild(os_elem)
 
+        if len(self.script_results) > 0:
+            hostscript_elem = document.createElement(u"hostscript")
+            for sr in self.script_results:
+                hostscript_elem.appendChild(sr.to_dom_fragment(document))
+            elem.appendChild(hostscript_elem)
+
         frag.appendChild(elem)
         return frag
 
@@ -244,6 +251,7 @@ class Port(object):
         self.spec = spec
         self.state = state
         self.service = Service()
+        self.script_results = []
 
     def state_string(self):
         if self.state is None:
@@ -258,7 +266,8 @@ class Port(object):
         d = cmp(self.spec, other.spec)
         if d != 0:
             return d
-        return cmp((self.spec, self.service), (other.spec, other.service))
+        return cmp((self.spec, self.service, self.script_results),
+            (other.spec, other.service, other.script_results))
 
     def to_dom_fragment(self, document):
         frag = document.createDocumentFragment()
@@ -270,6 +279,8 @@ class Port(object):
             state_elem.setAttribute(u"state", self.state)
             elem.appendChild(state_elem)
         elem.appendChild(self.service.to_dom_fragment(document))
+        for sr in self.script_results:
+            elem.appendChild(sr.to_dom_fragment(document))
         frag.appendChild(elem)
         return frag
 
@@ -335,6 +346,39 @@ class Service(object):
             frag.appendChild(elem)
         return frag
 
+class ScriptResult(object):
+    def __init__(self):
+        self.id = None
+        self.output = None
+
+    def __eq__(self, other):
+        return self.id == other.id and self.output == other.output
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __cmp__(self, other):
+        return cmp((self.id, self.output), (other.id, other.output))
+
+    def get_lines(self):
+        result = []
+        lines = self.output.splitlines()
+        if len(lines) > 0:
+            lines[0] = self.id + u": " + lines[0]
+        for line in lines[:-1]:
+            result.append(u"|  " + line)
+        if len(lines) > 0:
+            result.append(u"|_ " + lines[-1])
+        return result
+
+    def to_dom_fragment(self, document):
+        frag = document.createDocumentFragment()
+        elem = document.createElement(u"script")
+        elem.setAttribute(u"id", self.id)
+        elem.setAttribute(u"output", self.output)
+        frag.appendChild(elem)
+        return frag
+
 def format_banner(scan):
     """Format a startup banner more or less like Nmap does."""
     parts = [u"Nmap"]
@@ -402,6 +446,8 @@ class ScanDiff(object):
         document.writexml(f, addindent = u"  ", newl = u"\n", encoding = "UTF-8")
         document.unlink()
 
+    cost = property(lambda self: sum([hd.cost for hd in self.host_diffs.values()]))
+
 class HostDiff(object):
     """A diff of two Hosts. It contains the two hosts, variables describing what
     changed, and a list of PortDiffs and OS differences."""
@@ -415,6 +461,7 @@ class HostDiff(object):
         self.ports = []
         self.port_diffs = {}
         self.os_diffs = []
+        self.script_result_diffs = []
         self.cost = 0
 
         self.diff()
@@ -457,6 +504,9 @@ class HostDiff(object):
             self.extraports_changed = True
             self.cost += 1
 
+        self.script_result_diffs = ScriptResultDiff.diff_lists(self.host_a.script_results, self.host_b.script_results)
+        self.cost += len(self.script_result_diffs)
+
     def include_diff(self, diff):
         # Don't include the diff if the states are only extraports. Include all
         # diffs, even those with cost == 0, in verbose mode.
@@ -510,17 +560,8 @@ class HostDiff(object):
         port_table.append((mark, u"PORT", u"STATE", u"SERVICE", u"VERSION"))
 
         for port in self.ports:
-            diff = self.port_diffs[port]
-            port_a = diff.port_a
-            port_b = diff.port_b
-            if diff.cost == 0:
-                if verbose:
-                    port_table.append((u" ", port.spec_string(), port.state_string(), port.service.name_string(), port.service.version_string()))
-            else:
-                if not host_a.is_extraports(port_a.state):
-                    port_table.append((u"-", port_a.spec_string(), port_a.state_string(), port_a.service.name_string(), port_a.service.version_string()))
-                if not host_a.is_extraports(port_b.state):
-                    port_table.append((u"+", port_b.spec_string(), port_b.state_string(), port_b.service.name_string(), port_b.service.version_string()))
+            port_diff = self.port_diffs[port]
+            port_diff.append_to_port_table(port_table, host_a, host_b)
 
         if len(port_table) > 1:
             print >> f, port_table
@@ -546,6 +587,19 @@ class HostDiff(object):
                     for i in range(i1, i2):
                         print >> f, "   %s" % host_a.os[i]
 
+        table = Table(u"*")
+        for sr_diff in self.script_result_diffs:
+            sr_diff.append_to_port_table(table)
+        if len(table) > 0:
+            print >> f
+            if len(host_b.script_results) == 0:
+                print >> f, u"-Host script results:"
+            elif len(host_a.script_results) == 0:
+                print >> f, u"+Host script results:"
+            else:
+                print >> f, u" Host script results:"
+            print >> f, table
+
     def to_dom_fragment(self, document):
         host_a = self.host_a
         host_b = self.host_b
@@ -658,6 +712,28 @@ class HostDiff(object):
             if os_elem.hasChildNodes():
                 host_elem.appendChild(os_elem)
 
+        # Host script changes.
+        if len(self.script_result_diffs) > 0 or verbose:
+            hostscript_elem = document.createElement(u"hostscript")
+            if len(host_a.script_results) == 0 and len(host_b.script_results) == 0:
+                pass
+            elif len(host_b.script_results) == 0:
+                a_elem = document.createElement(u"a")
+                for sr in host_a.script_results:
+                    a_elem.appendChild(sr.to_dom_fragment(document))
+                a_elem.appendChild(hostscript_elem)
+                host_elem.appendChild(a_elem)
+            elif len(host_a.script_results) == 0:
+                b_elem = document.createElement(u"b")
+                for sr in host_b.script_results:
+                    b_elem.appendChild(sr.to_dom_fragment(document))
+                b_elem.appendChild(hostscript_elem)
+                host_elem.appendChild(b_elem)
+            else:
+                for sr_diff in self.script_result_diffs:
+                    hostscript_elem.appendChild(sr_diff.to_dom_fragment(document))
+                host_elem.appendChild(hostscript_elem)
+
         hostdiff_elem.appendChild(host_elem)
 
         return frag
@@ -668,6 +744,7 @@ class PortDiff(object):
     def __init__(self, port_a, port_b):
         self.port_a = port_a
         self.port_b = port_b
+        self.script_result_diffs = []
         self.cost = 0
 
         self.diff()
@@ -682,18 +759,139 @@ class PortDiff(object):
         if self.port_a.service != self.port_b.service:
             self.cost += 1
 
+        self.script_result_diffs = ScriptResultDiff.diff_lists(self.port_a.script_results, self.port_b.script_results)
+        self.cost += len(self.script_result_diffs)
+
+    # PortDiffs are inserted into a Table and then printed, not printed out
+    # directly. That's why this class has append_to_port_table instead of
+    # print_text.
+    def append_to_port_table(self, table, host_a, host_b):
+        """Append this port diff to a Table containing five columns:
+            +- PORT STATE SERVICE VERSION
+        The "+-" stands for the diff indicator column."""
+        a_columns = [self.port_a.spec_string(),
+            self.port_a.state_string(),
+            self.port_a.service.name_string(),
+            self.port_a.service.version_string()]
+        b_columns = [self.port_b.spec_string(),
+            self.port_b.state_string(),
+            self.port_b.service.name_string(),
+            self.port_b.service.version_string()]
+        if a_columns == b_columns:
+            if verbose or self.script_result_diffs > 0:
+                table.append([u" "] + a_columns)
+        else:
+            if not host_a.is_extraports(self.port_a.state):
+                table.append([u"-"] + a_columns)
+            if not host_b.is_extraports(self.port_b.state):
+                table.append([u"+"] + b_columns)
+
+        for sr_diff in self.script_result_diffs:
+            sr_diff.append_to_port_table(table)
+
     def to_dom_fragment(self, document):
         frag = document.createDocumentFragment()
         portdiff_elem = document.createElement(u"portdiff")
         frag.appendChild(portdiff_elem)
-        a_elem = document.createElement(u"a")
-        b_elem = document.createElement(u"b")
-        portdiff_elem.appendChild(a_elem)
-        portdiff_elem.appendChild(b_elem)
+        if self.port_a.spec == self.port_b.spec and self.port_a.state == self.port_b.state:
+            port_elem = document.createElement(u"port")
+            port_elem.setAttribute(u"portid", unicode(self.port_a.spec[0]))
+            port_elem.setAttribute(u"protocol", self.port_a.spec[1])
+            if self.port_a.state is not None:
+                state_elem = document.createElement(u"state")
+                state_elem.setAttribute(u"state", self.port_a.state)
+                port_elem.appendChild(state_elem)
+            if self.port_a.service == self.port_b.service:
+                port_elem.appendChild(self.port_a.service.to_dom_fragment(document))
+            else:
+                a_elem = document.createElement(u"a")
+                a_elem.appendChild(self.port_a.service.to_dom_fragment(document))
+                port_elem.appendChild(a_elem)
+                b_elem = document.createElement(u"b")
+                b_elem.appendChild(self.port_b.service.to_dom_fragment(document))
+                port_elem.appendChild(b_elem)
+            for sr_diff in self.script_result_diffs:
+                port_elem.appendChild(sr_diff.to_dom_fragment(document))
+            portdiff_elem.appendChild(port_elem)
+        else:
+            a_elem = document.createElement(u"a")
+            a_elem.appendChild(self.port_a.to_dom_fragment(document))
+            portdiff_elem.appendChild(a_elem)
+            b_elem = document.createElement(u"b")
+            b_elem.appendChild(self.port_b.to_dom_fragment(document))
+            portdiff_elem.appendChild(b_elem)
 
-        a_elem.appendChild(self.port_a.to_dom_fragment(document))
-        b_elem.appendChild(self.port_b.to_dom_fragment(document))
+        return frag
 
+class ScriptResultDiff(object):
+    def __init__(self, sr_a, sr_b):
+        """One of sr_a and sr_b may be None."""
+        self.sr_a = sr_a
+        self.sr_b = sr_b
+
+    def diff_lists(a, b):
+        """Return a list of ScriptResultDiffs from two sorted lists of
+        ScriptResults."""
+        diffs = []
+        i = 0
+        j = 0
+        # This algorithm is like a merge of sorted lists.
+        while i < len(a) and j < len(b):
+            if a[i].id < b[j].id:
+                diffs.append(ScriptResultDiff(a[i], None))
+                i += 1
+            elif a[i].id > b[j].id:
+                diffs.append(ScriptResultDiff(None, b[j]))
+                j += 1
+            else:
+                if a[i].output != b[j].output or verbose:
+                    diffs.append(ScriptResultDiff(a[i], b[j]))
+                i += 1
+                j += 1
+        while i < len(a):
+            diffs.append(ScriptResultDiff(a[i], None))
+            i += 1
+        while j < len(b):
+            diffs.append(ScriptResultDiff(None, b[j]))
+            j += 1
+        return diffs
+    diff_lists = staticmethod(diff_lists)
+
+    # Script result diffs are appended to a port table rather than being printed
+    # directly, so append_to_port_table exists instead of print_text.
+    def append_to_port_table(self, table):
+        a_lines = []
+        b_lines = []
+        if self.sr_a is not None:
+            a_lines = self.sr_a.get_lines()
+        if self.sr_b is not None:
+            b_lines = self.sr_b.get_lines()
+        if a_lines != b_lines or verbose:
+            diffs = difflib.SequenceMatcher(None, a_lines, b_lines)
+            for op, i1, i2, j1, j2 in diffs.get_opcodes():
+                if op == "replace" or op == "delete":
+                    for k in range(i1, i2):
+                        table.append_raw(u"-" + a_lines[k])
+                if op == "replace" or op == "insert":
+                    for k in range(j1, j2):
+                        table.append_raw(u"+" + b_lines[k])
+                if op == "equal":
+                    for k in range(i1, i2):
+                        table.append_raw(u" " + a_lines[k])
+
+    def to_dom_fragment(self, document):
+        frag = document.createDocumentFragment()
+        if self.sr_a is not None and self.sr_b is not None and self.sr_a == self.sr_b:
+            frag.appendChild(self.sr_a.to_dom_fragment(document))
+        else:
+            if self.sr_a is not None:
+                a_elem = document.createElement(u"a")
+                a_elem.appendChild(self.sr_a.to_dom_fragment(document))
+                frag.appendChild(a_elem)
+            if self.sr_b is not None:
+                b_elem = document.createElement(u"b")
+                b_elem.appendChild(self.sr_b.to_dom_fragment(document))
+                frag.appendChild(b_elem)
         return frag
 
 class Table(object):
@@ -739,6 +937,10 @@ class Table(object):
             strings.append(s)
         self.rows.append(strings)
 
+    def append_raw(self, s):
+        """Append a raw string for a row that is not formatted into columns."""
+        self.rows.append(s)
+
     def __len__(self):
         return len(self.rows)
 
@@ -747,39 +949,18 @@ class Table(object):
         for row in self.rows:
             parts = [self.prefix]
             i = 0
-            while i < len(row):
-                parts.append(row[i].ljust(self.widths[i]))
-                if i < len(self.padding):
-                    parts.append(self.padding[i])
-                i += 1
-            lines.append(u"".join(parts).rstrip())
+            if isinstance(row, basestring):
+                # A raw string.
+                lines.append(row)
+            else:
+                while i < len(row):
+                    parts.append(row[i].ljust(self.widths[i]))
+                    if i < len(self.padding):
+                        parts.append(self.padding[i])
+                    i += 1
+                lines.append(u"".join(parts).rstrip())
         return u"\n".join(lines)
 
-def parse_port_list(port_list):
-    """Parse a port list like
-    "1-1027,1029-1033,1040,1043,1050,1058-1059,1067-1068,1076,1080,1083-1084"
-    into a list of numbers. Raises ValueError if the port list is somehow
-    invalid."""
-    result = set()
-    if port_list == u"":
-        return list(result)
-    chunks = port_list.split(u",")
-    for chunk in chunks:
-        if u"-" in chunk:
-            start, end = chunk.split(u"-")
-            start = int(start)
-            end = int(end)
-            if start >= end:
-                raise ValueError(u"In range %s, start must be less than end." % chunk)
-        else:
-            start = int(chunk)
-            end = start
-        for p in range(start, end + 1):
-            result.add(p)
-    result = list(result)
-    result.sort()
-    return result
-
 def warn(str):
     """Print a warning to stderr."""
     print >> sys.stderr, str
@@ -795,7 +976,6 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
         # popping on end.
         self.element_stack = []
 
-        self.scanned_ports = {}
         self.current_host = None
         self.current_port = None
 
@@ -828,25 +1008,6 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
                 start_timestamp = int(attrs.get(u"start"))
                 self.scan.start_date = datetime.datetime.fromtimestamp(start_timestamp)
             self.scan.version = attrs.get(u"version")
-        elif name == u"scaninfo":
-            assert self.parent_element() == u"nmaprun"
-            try:
-                protocol = attrs[u"protocol"]
-            except KeyError:
-                warn(u"%s element missing the \"protocol\" attribute; skipping." % name)
-                return
-            try:
-                services_str = attrs[u"services"]
-            except KeyError:
-                warn(u"%s element missing the \"services\" attribute; skipping." % name)
-                return
-            try:
-                services = parse_port_list(services_str)
-            except ValueError:
-                warn(u"Services list \"%s\" cannot be parsed; skipping.")
-                return
-            assert protocol not in self.scanned_ports
-            self.scanned_ports[protocol] = services
         elif name == u"host":
             assert self.parent_element() == u"nmaprun"
             self.current_host = Host()
@@ -937,6 +1098,26 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
             self.current_port.service.version = attrs.get(u"version")
             self.current_port.service.extrainfo = attrs.get(u"extrainfo")
             self.current_port.service.tunnel = attrs.get(u"tunnel")
+        elif name == u"script":
+            assert self.current_host is not None
+            result = ScriptResult()
+            try:
+                result.id = attrs[u"id"]
+            except KeyError:
+                warn(u"%s element of host %s missing the \"id\" attribute; skipping." % (name, self.current_host.format_name()))
+                return
+            try:
+                result.output = attrs[u"output"]
+            except KeyError:
+                warn(u"%s element of host %s missing the \"output\" attribute; skipping." % (name, self.current_host.format_name()))
+                return
+            if self.parent_element() == u"hostscript":
+                self.current_host.script_results.append(result)
+            elif self.parent_element() == u"port":
+                self.current_port.script_results.append(result)
+            else:
+                warn(u"%s element of host %s not inside hostscript or port element; ignoring." % (name, self.current_host.format_name()))
+                return
         elif name == u"osmatch":
             assert self.parent_element() == u"os"
             assert self.current_host is not None
@@ -951,23 +1132,11 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
                 self.scan.end_date = datetime.datetime.fromtimestamp(end_timestamp)
 
     def endElementAux(self, name):
-        if name == u"nmaprun":
-            self.scanned_ports = None
-        elif name == u"host":
-            if len(self.current_host.extraports) == 1:
-                # We can infer the state of unlisted ports.
-                extraports_state = self.current_host.extraports.keys()[0]
-                known_specs = self.current_host.ports.keys()
-                for protocol in self.scanned_ports:
-                    for portid in self.scanned_ports[protocol]:
-                        spec = portid, protocol
-                        if spec in known_specs:
-                            continue
-                        assert spec not in self.current_host.ports
-                        self.current_host.add_port(Port(spec, state = extraports_state))
-
+        if name == u"host":
+            self.current_host.script_results.sort()
             self.current_host = None
         elif name == u"port":
+            self.current_port.script_results.sort()
             self.current_port = None
 
 def usage():
@@ -983,10 +1152,14 @@ service and OS detection.
   --xml          display output in XML format\
 """ % sys.argv[0]
 
+EXIT_EQUAL = 0
+EXIT_DIFFERENT = 1
+EXIT_ERROR = 2
+
 def usage_error(msg):
     print >> sys.stderr, u"%s: %s" % (sys.argv[0], msg)
     print >> sys.stderr, u"Try '%s -h' for help." % sys.argv[0]
-    sys.exit(1)
+    sys.exit(EXIT_ERROR)
 
 def main():
     global verbose
@@ -1032,5 +1205,17 @@ def main():
     elif output_format == "xml":
         diff.print_xml()
 
+    if diff.cost == 0:
+        return EXIT_EQUAL
+    else:
+        return EXIT_DIFFERENT
+
+# Catch uncaught exceptions so they can produce an exit code of 2 (EXIT_ERROR),
+# not 1 like they would by default.
+def excepthook(type, value, tb):
+    sys.__excepthook__(type, value, tb)
+    sys.exit(EXIT_ERROR)
+
 if __name__ == "__main__":
-    main()
+    sys.excepthook = excepthook
+    sys.exit(main())
